@@ -7,7 +7,9 @@ use Glib qw(TRUE FALSE);   # To get TRUE and FALSE
 use Sane 0.05;             # To get SANE_NAME_PAGE_WIDTH & SANE_NAME_PAGE_HEIGHT
 use Gscan2pdf::Dialog;
 use Data::Dumper;
+use Storable qw(dclone);
 use feature 'switch';
+use Gscan2pdf::Scanner::Options;
 my (
     $_MAX_PAGES,        $_MAX_INCREMENT, $_DOUBLE_INCREMENT,
     $_CANVAS_SIZE,      $_CANVAS_BORDER, $_CANVAS_POINT_SIZE,
@@ -16,10 +18,9 @@ my (
 
 # need to register this with Glib before we can use it below
 BEGIN {
-    use Gscan2pdf::Scanner::Options;
-    Glib::Type->register_enum( 'Gscan2pdf::Scanner::Dialog::Side',
+    Glib::Type->register_enum( 'Gscan2pdf::Dialog::Scan::Side',
         qw(facing reverse) );
-    Glib::Type->register_enum( 'Gscan2pdf::Scanner::Dialog::Sided',
+    Glib::Type->register_enum( 'Gscan2pdf::Dialog::Scan::Sided',
         qw(single double) );
     use Readonly;
     Readonly $_MAX_PAGES         => 9999;
@@ -180,28 +181,28 @@ use Glib::Object::Subclass Gscan2pdf::Dialog::, signals => {
         [qw/readable writable/]    # flags
     ),
     Glib::ParamSpec->enum(
-        'sided',                                # name
-        'Sided',                                # nickname
-        'Either single or double',              # blurb
-        'Gscan2pdf::Scanner::Dialog::Sided',    # type
-        'single',                               # default
-        [qw/readable writable/]                 # flags
+        'sided',                             # name
+        'Sided',                             # nickname
+        'Either single or double',           # blurb
+        'Gscan2pdf::Dialog::Scan::Sided',    # type
+        'single',                            # default
+        [qw/readable writable/]              # flags
     ),
     Glib::ParamSpec->enum(
-        'side-to-scan',                         # name
-        'Side to scan',                         # nickname
-        'Either facing or reverse',             # blurb
-        'Gscan2pdf::Scanner::Dialog::Side',     # type
-        'facing',                               # default
-        [qw/readable writable/]                 # flags
+        'side-to-scan',                      # name
+        'Side to scan',                      # nickname
+        'Either facing or reverse',          # blurb
+        'Gscan2pdf::Dialog::Scan::Side',     # type
+        'facing',                            # default
+        [qw/readable writable/]              # flags
     ),
     Glib::ParamSpec->object(
-        'available-scan-options',               # name
-        'Scan options available',               # nickname
+        'available-scan-options',            # name
+        'Scan options available',            # nickname
         'Scan options currently available, whether active, selected, or not'
-        ,                                       # blurb
-        'Gscan2pdf::Scanner::Options',          # package
-        [qw/readable writable/]                 # flags
+        ,                                    # blurb
+        'Gscan2pdf::Scanner::Options',       # package
+        [qw/readable writable/]              # flags
     ),
     Glib::ParamSpec->scalar(
         'current-scan-options',                               # name
@@ -233,7 +234,7 @@ use Glib::Object::Subclass Gscan2pdf::Dialog::, signals => {
     ),
   ];
 
-our $VERSION = '1.5.0';
+our $VERSION = '1.5.1';
 
 my ( $d, $d_sane, $logger, $tooltips );
 my $tolerance = 1;
@@ -248,34 +249,7 @@ sub INIT_INSTANCE {
     $d      = Locale::gettext->domain(Glib::get_application_name);
     $d_sane = Locale::gettext->domain('sane-backends');
 
-    # device list
-    $self->{hboxd} = Gtk2::HBox->new;
-    my $labeld = Gtk2::Label->new( $d->get('Device') );
-    $self->{hboxd}->pack_start( $labeld, FALSE, FALSE, 0 );
-    $self->{combobd} = Gtk2::ComboBox->new_text;
-    $self->{combobd}->append_text( $d->get('Rescan for devices') );
-
-    $self->{combobd_changed_signal} = $self->{combobd}->signal_connect(
-        changed => sub {
-            my $index       = $self->{combobd}->get_active;
-            my $device_list = $self->get('device-list');
-            if ( $index > $#{$device_list} ) {
-                $self->{combobd}->hide;
-                $labeld->hide;
-                $self->set( 'device', undef )
-                  ;    # to make sure that the device is reloaded
-                $self->get_devices;
-            }
-            elsif ( $index > $_NO_INDEX ) {
-                $self->set( 'device', $device_list->[$index]{name} );
-            }
-        }
-    );
-    $self->signal_connect( 'changed-device' => \&_changed_device_callback );
-    $tooltips->set_tip( $self->{combobd},
-        $d->get('Sets the device to be used for the scan') );
-    $self->{hboxd}->pack_end( $self->{combobd}, FALSE, FALSE, 0 );
-    $vbox->pack_start( $self->{hboxd}, FALSE, FALSE, 0 );
+    $self->_add_device_combobox($vbox);
 
     # Notebook to collate options
     $self->{notebook} = Gtk2::Notebook->new;
@@ -523,30 +497,7 @@ sub INIT_INSTANCE {
 
     # Save button
     my $vbutton = Gtk2::Button->new_from_stock('gtk-save');
-    $vbutton->signal_connect(
-        clicked => sub {
-            my $dialog = Gtk2::Dialog->new(
-                $d->get('Name of scan profile'), $self,
-                'destroy-with-parent',
-                'gtk-save'   => 'ok',
-                'gtk-cancel' => 'cancel'
-            );
-            my $hbox  = Gtk2::HBox->new;
-            my $label = Gtk2::Label->new( $d->get('Name of scan profile') );
-            $hbox->pack_start( $label, FALSE, FALSE, 0 );
-            my $entry = Gtk2::Entry->new;
-            $entry->set_activates_default(TRUE);
-            $hbox->pack_end( $entry, TRUE, TRUE, 0 );
-            $dialog->vbox->add($hbox);
-            $dialog->set_default_response('ok');
-            $dialog->show_all;
-
-            if ( $dialog->run eq 'ok' and $entry->get_text !~ /^\s*$/xsm ) {
-                $self->save_current_profile( $entry->get_text );
-            }
-            $dialog->destroy;
-        }
-    );
+    $vbutton->signal_connect( clicked => \&_save_profile_callback, $self );
     $hboxsp->pack_start( $vbutton, TRUE, TRUE, 0 );
 
     # Delete button
@@ -581,22 +532,108 @@ sub INIT_INSTANCE {
     return $self;
 }
 
-sub _changed_device_callback {
-    my ( $self, $device ) = @_;
-    my $device_list = $self->get('device-list');
-    if ( defined $device and $device ne $EMPTY ) {
-        for ( @{$device_list} ) {
-            if ( $_->{name} eq $device ) {
-                Gscan2pdf::Dialog::Scan::set_combobox_by_text( $self->{combobd},
-                    $_->{label} );
-                $self->scan_options;
-                return;
+sub _add_device_combobox {
+    my ( $self, $vbox ) = @_;
+    $self->{hboxd} = Gtk2::HBox->new;
+    my $labeld = Gtk2::Label->new( $d->get('Device') );
+    $self->{hboxd}->pack_start( $labeld, FALSE, FALSE, 0 );
+    $self->{combobd} = Gtk2::ComboBox->new_text;
+    $self->{combobd}->append_text( $d->get('Rescan for devices') );
+
+    $self->{combobd_changed_signal} = $self->{combobd}->signal_connect(
+        changed => sub {
+            my $index       = $self->{combobd}->get_active;
+            my $device_list = $self->get('device-list');
+            if ( $index > $#{$device_list} ) {
+                $self->{combobd}->hide;
+                $labeld->hide;
+                $self->set( 'device', undef )
+                  ;    # to make sure that the device is reloaded
+                $self->get_devices;
+            }
+            elsif ( $index > $_NO_INDEX ) {
+                $self->set( 'device', $device_list->[$index]{name} );
             }
         }
+    );
+    $self->signal_connect(
+        'changed-device' => sub {
+            my ( $self, $device ) = @_;
+            my $device_list = $self->get('device-list');
+            if ( defined $device and $device ne $EMPTY ) {
+                for ( @{$device_list} ) {
+                    if ( $_->{name} eq $device ) {
+                        Gscan2pdf::Dialog::Scan::set_combobox_by_text(
+                            $self->{combobd}, $_->{label} );
+                        $self->scan_options;
+                        return;
+                    }
+                }
+            }
+            else {
+                $self->{combobd}->set_active($_NO_INDEX);
+            }
+        }
+    );
+    $tooltips->set_tip( $self->{combobd},
+        $d->get('Sets the device to be used for the scan') );
+    $self->{hboxd}->pack_end( $self->{combobd}, FALSE, FALSE, 0 );
+    $vbox->pack_start( $self->{hboxd}, FALSE, FALSE, 0 );
+    return;
+}
+
+sub _save_profile_callback {
+    my ( $widget, $parent ) = @_;
+    my $dialog = Gtk2::Dialog->new(
+        $d->get('Name of scan profile'), $parent,
+        'destroy-with-parent',
+        'gtk-save'   => 'ok',
+        'gtk-cancel' => 'cancel'
+    );
+    my $hbox  = Gtk2::HBox->new;
+    my $label = Gtk2::Label->new( $d->get('Name of scan profile') );
+    $hbox->pack_start( $label, FALSE, FALSE, 0 );
+    my $entry = Gtk2::Entry->new;
+    $entry->set_activates_default(TRUE);
+    $hbox->pack_end( $entry, TRUE, TRUE, 0 );
+    $dialog->vbox->add($hbox);
+    $dialog->set_default_response('ok');
+    $dialog->show_all;
+    my $flag = TRUE;
+
+    while ($flag) {
+        if ( $dialog->run eq 'ok' ) {
+            my $name = $entry->get_text;
+            if ( $name !~ /^\s*$/xsm ) {
+                if ( defined $parent->{profiles}{$name} ) {
+                    my $warning = sprintf
+                      $d->get("Profile '%s' exists. Overwrite?"),
+                      $name;
+                    my $dialog2 = Gtk2::Dialog->new(
+                        $warning, $dialog, 'destroy-with-parent',
+                        'gtk-ok'     => 'ok',
+                        'gtk-cancel' => 'cancel'
+                    );
+                    $label = Gtk2::Label->new($warning);
+                    $dialog2->vbox->add($label);
+                    $label->show;
+                    if ( $dialog2->run eq 'ok' ) {
+                        $parent->save_current_profile( $entry->get_text );
+                        $flag = FALSE;
+                    }
+                    $dialog2->destroy;
+                }
+                else {
+                    $parent->save_current_profile( $entry->get_text );
+                    $flag = FALSE;
+                }
+            }
+        }
+        else {
+            $flag = FALSE;
+        }
     }
-    else {
-        $self->{combobd}->set_active($_NO_INDEX);
-    }
+    $dialog->destroy;
     return;
 }
 
@@ -626,38 +663,10 @@ sub SET_PROPERTY {
         my $callback = FALSE;
         given ($name) {
             when ('allow_batch_flatbed') {
-                $self->{$name} = $newval;
-                if ($newval) {
-                    $self->{framen}->set_sensitive(TRUE);
-                }
-                else {
-                    my $options = $self->get('available-scan-options');
-                    if ( defined $options
-                        and $options->by_name('source')->{val} =~
-                        /flatbed/xsmi )
-                    {
-                        $self->{framen}->set_sensitive(FALSE);
-
-                        # emits changed-num-pages signal, allowing us to test
-                        # for $self->{framen}->set_sensitive(FALSE)
-                        $self->set( 'num-pages', 1 );
-                    }
-                }
+                $self->_set_allow_batch_flatbed( $name, $newval );
             }
             when ('available_scan_options') {
-                $self->{$name} = $newval;
-                if ( not $self->get('allow-batch-flatbed')
-                    and $newval->by_name('source')->{val} =~ /flatbed/xsmi )
-                {
-                    if ( $self->get('num-pages') != 1 ) {
-                        $self->set( 'num-pages', 1 );
-                    }
-                    $self->{framen}->set_sensitive(FALSE);
-                }
-                else {
-                    $self->{framen}->set_sensitive(TRUE);
-                }
-                $self->signal_emit('reloaded-scan-options')
+                $self->_set_available_scan_options( $name, $newval );
             }
             when ('device') {
                 $self->{$name} = $newval;
@@ -670,18 +679,7 @@ sub SET_PROPERTY {
                 $self->signal_emit( 'changed-device-list', $newval )
             }
             when ('num_pages') {
-                my $options = $self->get('available-scan-options');
-                if (
-                       $newval == 1
-                    or $self->get('allow-batch-flatbed')
-                    or ( defined $options
-                        and $options->by_name('source')->{val} !~
-                        /flatbed/xsmi )
-                  )
-                {
-                    $self->{$name} = $newval;
-                    $self->signal_emit( 'changed-num-pages', $newval );
-                }
+                $self->_set_num_pages( $name, $newval );
             }
             when ('page_number_start') {
                 $self->{$name} = $newval;
@@ -735,6 +733,7 @@ sub SET_PROPERTY {
                     }
                 );
                 $self->set_paper($newval);
+                $self->{current_scan_options}{frontend}{$name} = $newval;
             }
             when ('paper_formats') {
                 $self->{$name} = $newval;
@@ -770,6 +769,60 @@ sub SET_PROPERTY {
     return;
 }
 
+sub _set_allow_batch_flatbed {
+    my ( $self, $name, $newval ) = @_;
+    $self->{$name} = $newval;
+    if ($newval) {
+        $self->{framen}->set_sensitive(TRUE);
+    }
+    else {
+        my $options = $self->get('available-scan-options');
+        if ( defined $options
+            and $options->by_name('source')->{val} =~ /flatbed/xsmi )
+        {
+            $self->{framen}->set_sensitive(FALSE);
+
+            # emits changed-num-pages signal, allowing us to test
+            # for $self->{framen}->set_sensitive(FALSE)
+            $self->set( 'num-pages', 1 );
+        }
+    }
+    return;
+}
+
+sub _set_available_scan_options {
+    my ( $self, $name, $newval ) = @_;
+    $self->{$name} = $newval;
+    if ( not $self->get('allow-batch-flatbed')
+        and $newval->by_name('source')->{val} =~ /flatbed/xsmi )
+    {
+        if ( $self->get('num-pages') != 1 ) { $self->set( 'num-pages', 1 ) }
+        $self->{framen}->set_sensitive(FALSE);
+    }
+    else {
+        $self->{framen}->set_sensitive(TRUE);
+    }
+    $self->signal_emit('reloaded-scan-options');
+    return;
+}
+
+sub _set_num_pages {
+    my ( $self, $name, $newval ) = @_;
+    my $options = $self->get('available-scan-options');
+    if (
+           $newval == 1
+        or $self->get('allow-batch-flatbed')
+        or ( defined $options
+            and $options->by_name('source')->{val} !~ /flatbed/xsmi )
+      )
+    {
+        $self->{$name} = $newval;
+        $self->{current_scan_options}{frontend}{$name} = $newval;
+        $self->signal_emit( 'changed-num-pages', $newval );
+    }
+    return;
+}
+
 sub show {
     my $self = shift;
     $self->signal_chain_from_overridden;
@@ -863,7 +916,7 @@ sub pack_widget {
     my ( $self, $widget, $data ) = @_;
     my ( $options, $opt, $hbox, $hboxp ) = @{$data};
     if ( defined $widget ) {
-        $opt->{widget} = $widget;
+        $self->{option_widgets}{ $opt->{name} } = $widget;
         if ( $opt->{type} == SANE_TYPE_BUTTON or $opt->{max_values} > 1 ) {
             $hbox->pack_end( $widget, TRUE, TRUE, 0 );
         }
@@ -874,7 +927,7 @@ sub pack_widget {
 
         # Look-up to hide/show the box if necessary
         if ( $self->_geometry_option($opt) ) {
-            $options->{box}{ $opt->{name} } = $hbox;
+            $self->{geometry_boxes}{ $opt->{name} } = $hbox;
         }
 
         $self->create_paper_widget( $options, $hboxp );
@@ -899,13 +952,13 @@ sub update_options {
     my $num_dev_options = $new_options->num_options;
     my $options         = $self->get('available-scan-options');
     for ( 1 .. $num_dev_options - 1 ) {
-        my $widget = $options->by_index($_)->{widget};
+        my $opt    = $options->by_index($_);
+        my $widget = $self->{option_widgets}{ $opt->{name} };
 
         # could be undefined for !($new_opt->{cap} & SANE_CAP_SOFT_DETECT)
         if ( not defined $widget ) { next }
 
         my $new_opt = $new_options->by_index($_);
-        my $opt     = $options->by_index($_);
         if ( $new_opt->{name} ne $opt->{name} ) {
             $logger->error(
 'Error updating options: reloaded options are numbered differently'
@@ -923,7 +976,6 @@ sub update_options {
         # Block the signal handler for the widget to prevent infinite
         # loops of the widget updating the option, updating the widget, etc.
         $widget->signal_handler_block( $widget->{signal} );
-        $new_opt->{widget} = $widget;
         $opt = $new_opt;
         my $value = $opt->{val};
 
@@ -1213,26 +1265,6 @@ sub edit_paper {
     return;
 }
 
-sub _clone_profile {
-    my ($profile) = @_;
-    my @clone;
-    if ( ref($profile) eq 'ARRAY' ) {
-        for ( @{$profile} ) {
-            my ( $key, $val ) = each %{$_};
-            if ( defined $key ) { push @clone, { $key => $val } }
-        }
-    }
-    elsif ( ref($profile) eq 'HASH' ) {
-
-        # If the profile is a hash, the order is undefined.
-        # Sort it to be consistent for tests.
-        for my $key ( sort keys %{$profile} ) {
-            push @clone, { $key => $profile->{$key} };
-        }
-    }
-    return \@clone;
-}
-
 # keeping this as a separate sub allows us to test it
 sub save_current_profile {
     my ( $self, $name ) = @_;
@@ -1248,7 +1280,10 @@ sub add_profile {
 
         # if we don't clone the profile,
         # we get strange action-at-a-distance problems
-        $self->{profiles}{$name} = _clone_profile($profile);
+        $self->{profiles}{$name} = dclone($profile);
+
+        _combobox_remove_item_by_text( $self->{combobsp}, $name );
+
         $self->{combobsp}->append_text($name);
         $logger->debug( "Saved profile '$name':",
             Dumper( $self->{profiles}{$name} ) );
@@ -1257,6 +1292,20 @@ sub add_profile {
     return;
 }
 
+sub _combobox_remove_item_by_text {
+    my ( $combobox, $text ) = @_;
+    if ( defined $text ) {
+        my $i = get_combobox_by_text( $combobox, $text );
+        if ( $i > $_NO_INDEX ) {
+            if ( $combobox->get_active == $i ) {
+                $combobox->set_active($_NO_INDEX);
+            }
+            $combobox->remove_text($i);
+        }
+    }
+    return;
+}
+
 sub set_profile {
     my ( $self, $name ) = @_;
     if ( defined $name and $name ne $EMPTY ) {
@@ -1271,11 +1320,18 @@ sub set_profile {
                 $self->signal_handler_disconnect($signal);
                 $self->{setting_profile} = FALSE;
 
+                # Having set all backend options, set the frontend options
+                if ( defined $self->{profiles}{$name}{frontend} ) {
+                    for my $key ( keys %{ $self->{profiles}{$name}{frontend} } )
+                    {
+                        $self->set( $key,
+                            $self->{profiles}{$name}{frontend}{$key} );
+                    }
+                }
+
                 # set property before emitting signal to ensure callbacks
                 # receive correct value
                 $self->{profile} = $name;
-                my $paper = $self->get_paper_by_geometry;
-                if ( defined $paper ) { $self->set( 'paper', $paper ) }
                 $self->signal_emit( 'changed-profile', $name );
             }
         );
@@ -1289,10 +1345,10 @@ sub set_profile {
         # but it shouldn't hurt, and otherwise, it fails t/0601_Dialog_Scan.t
 
         # I also don't understand why the $clone variable is necessary: why
-        # I can't simply pass the return value from _clone_profile() directly
+        # I can't simply pass the return value from dclone() directly
         # into set_current_scan_options(),
         # but otherwise it fails t/0610_Dialog_Scan.t
-        my $clone = _clone_profile( $self->{profiles}{$name} );
+        my $clone = dclone( $self->{profiles}{$name} );
         $self->set_current_scan_options($clone);
     }
 
@@ -1311,14 +1367,8 @@ sub set_profile {
 sub remove_profile {
     my ( $self, $name ) = @_;
     if ( defined $name and defined $self->{profiles}{$name} ) {
-        my $i = get_combobox_by_text( $self->{combobsp}, $name );
-        if ( $i > $_NO_INDEX ) {
-            if ( $self->{combobsp}->get_active == $i ) {
-                $self->{combobsp}->set_active($_NO_INDEX);
-            }
-            $self->{combobsp}->remove_text($i);
-            $self->signal_emit( 'removed-profile', $name );
-        }
+        _combobox_remove_item_by_text( $self->{combobsp}, $name );
+        $self->signal_emit( 'removed-profile', $name );
         delete $self->{profiles}{$name};
     }
     return;
@@ -1329,7 +1379,7 @@ sub remove_profile {
 sub build_profile {
     my ( $self, $profile, $option, $newval ) = @_;
     if ( $option->{val} != $newval ) {
-        push @{$profile}, { $option->{name} => $newval };
+        push @{ $profile->{backend} }, { $option->{name} => $newval };
     }
     return;
 }
@@ -1721,16 +1771,16 @@ sub update_graph {
 # Set options to profile referenced by hashref
 
 sub set_current_scan_options {
-    my ( $self, $profile ) = @_;
+    my ( $self, $options_arrayref ) = @_;
 
-    if ( not defined $profile ) { return }
+    if ( not defined $options_arrayref ) { return }
 
     # First clone the profile, as otherwise it would be self-modifying
-    my $clone = _clone_profile($profile);
+    my $clone = dclone($options_arrayref);
 
     # As scanimage and scanadf rename the geometry options,
     # we have to map them back to the original names
-    $self->map_geometry_names($clone);
+    $self->map_geometry_names( $clone->{backend} );
 
     # Give the GUI a chance to catch up between settings,
     # in case they have to be reloaded.
@@ -1741,24 +1791,18 @@ sub set_current_scan_options {
 
 sub _set_option_profile {
     my ( $self, $i, $profile ) = @_;
-    if ( $i < @{$profile} ) {
+    if ( $i < @{ $profile->{backend} } ) {
 
         # for reasons I don't understand, without walking the reference tree,
-        # parts of $profile are undef
-        Dumper( $profile->[$i] );
-        my ( $name, $val ) = each %{ $profile->[$i] };
-
-        if ( $name eq 'Paper size' ) {
-            $self->set( 'paper', $val );
-            $self->_set_option_profile( $i + 1, $profile );
-            return;
-        }
+        # parts of $profile->{backend} are undef
+        Dumper( $profile->{backend}[$i] );
+        my ( $name, $val ) = each %{ $profile->{backend}[$i] };
 
         my $options = $self->get('available-scan-options');
         my $opt     = $options->by_name($name);
         if ( not defined $opt or $opt->{cap} & SANE_CAP_INACTIVE ) {
             $logger->warn("Ignoring inactive option '$name'.");
-            splice @{$profile}, $i, 1;
+            splice @{ $profile->{backend} }, $i, 1;
             $self->_set_option_profile( $i, $profile );
             return;
         }
@@ -1773,7 +1817,7 @@ sub _set_option_profile {
         );
         $self->set_option( $opt, $val );
 
-        my $widget = $opt->{widget};
+        my $widget = $self->{option_widgets}{ $opt->{name} };
         if ( defined $widget ) {
             $logger->debug("Setting widget '$name' to '$val'.");
             $widget->signal_handler_block( $widget->{signal} );
@@ -1849,23 +1893,16 @@ sub make_progress_string {
 sub add_to_current_scan_options {
     my ( $self, $option, $val ) = @_;
     my $current = $self->{current_scan_options};
-    if ( not defined $current ) {
-        $current = [];
-    }
-
-    # Config::General flattens an array of hashes with 1 entry to a hash,
-    # so we must check for this
-    elsif ( ref($current) eq 'HASH' ) {
-        my %hash = %{$current};
-        $current = [ \%hash ];
+    if ( not defined $current->{backend} ) {
+        $current->{backend} = [];
     }
 
     # Cache option
-    push @{$current}, { $option->{name} => $val };
+    push @{ $current->{backend} }, { $option->{name} => $val };
 
     # Note any duplicate options, keeping only the last entry.
-    $self->{current_scan_options} =
-      Gscan2pdf::Scanner::Options::prune_duplicates($current);
+    $self->{current_scan_options}{backend} =
+      Gscan2pdf::Scanner::Options::prune_duplicates( $current->{backend} );
     return;
 }
 
