@@ -12,6 +12,8 @@ use Locale::gettext 1.05;    # For translations
 use feature 'switch';
 use List::MoreUtils qw{any};
 use Data::Dumper;
+use Readonly;
+Readonly my $LAST_PAGE => -1;
 
 my ( $d, $d_sane, $logger, $tooltips, $EMPTY, $COMMA );
 
@@ -77,7 +79,7 @@ use Glib::Object::Subclass Gscan2pdf::Dialog::Scan::, signals => {
     ),
   ];
 
-our $VERSION = '1.5.0';
+our $VERSION = '1.5.1';
 
 my $SANE_NAME_PAGE_HEIGHT = SANE_NAME_PAGE_HEIGHT;
 my $SANE_NAME_PAGE_WIDTH  = SANE_NAME_PAGE_WIDTH;
@@ -188,7 +190,7 @@ sub cache_key {
         Dumper( $self->{current_scan_options} );
 
         # grep the reload triggers from the current options
-        for ( @{ $self->{current_scan_options} } ) {
+        for ( @{ $self->{current_scan_options}{backend} } ) {
             my ( $key, $value ) = each %{$_};
             for ( @{$reload_triggers} ) {
                 if (/^$key$/ixsm) {
@@ -212,11 +214,13 @@ sub scan_options {
 
     # Remove any existing pages
     while ( $self->{notebook}->get_n_pages > 1 ) {
-
-        # -1 = last page
-        $self->{notebook}->remove_page(-1);  ## no critic (ProhibitMagicNumbers)
+        $self->{notebook}->remove_page($LAST_PAGE);
     }
 
+    # Remove lookups to geometry boxes and option widgets
+    delete $self->{geometry_boxes};
+    delete $self->{option_widgets};
+
     # Ghost the scan button whilst options being updated
     if ( defined $self->{sbutton} ) { $self->{sbutton}->set_sensitive(FALSE) }
 
@@ -248,7 +252,7 @@ sub scan_options {
         prefix           => $self->get('prefix'),
         frontend         => $self->get('frontend'),
         device           => $self->get('device'),
-        options          => $self->{current_scan_options},
+        options          => $self->{current_scan_options}{backend},
         started_callback => sub {
 
             # Set up ProgressBar
@@ -341,7 +345,7 @@ sub _initialise_options {    ## no critic (ProhibitExcessComplexity)
             $vbox = Gtk2::VBox->new;
             $self->{notebook}
               ->append_page( $vbox, $d_sane->get( $opt->{title} ) );
-            $opt->{widget} = $vbox;
+            $self->{option_widgets}{ $opt->{name} } = $vbox;
             next;
         }
 
@@ -506,8 +510,8 @@ sub _update_option_visibility {
         }
         my $container =
             $opt->{type} == SANE_TYPE_GROUP
-          ? $opt->{widget}
-          : $opt->{widget}->parent;
+          ? $self->{option_widgets}{ $opt->{name} }
+          : $self->{option_widgets}{ $opt->{name} }->parent;
         my $geometry = $self->_geometry_option($opt);
         if ($show) {
             $container->show_all;
@@ -519,8 +523,10 @@ sub _update_option_visibility {
                 and $options->{array}[$j]{type} != SANE_TYPE_GROUP )
             {
             }
-            if ( $j > 0 and not $options->{array}[$j]{widget}->visible ) {
-                my $group = $options->{array}[$j]{widget};
+            if ( $j > 0
+                and not $self->{option_widgets}{ $opt->{name} }->visible )
+            {
+                my $group = $self->{option_widgets}{ $opt->{name} };
                 if ( not $group->visible ) {
                     $group->remove($container);
                     my $move_paper =
@@ -556,12 +562,16 @@ sub _find_visible_group {
     my ( $self, $options, $option_number ) = @_;
     while (
         --$option_number > 0
-        and ( $options->{array}[$option_number]{type} != SANE_TYPE_GROUP
-            or ( not $options->{array}[$option_number]{widget}->visible ) )
+        and (
+            $options->{array}[$option_number]{type} != SANE_TYPE_GROUP
+            or (
+                not $self->{option_widgets}
+                { $options->{array}[$option_number]->{name} }->visible )
+        )
       )
     {
     }
-    return $options->{array}[$option_number]{widget}
+    return $self->{option_widgets}{ $options->{array}[$option_number]->{name} }
       if ( $option_number > 0 );
     return $self->{notebook}->get_nth_page(1);
 }
@@ -582,16 +592,16 @@ sub _geometry_option {
 # Return true if all the geometry widgets have been created
 
 sub _geometry_widgets_created {
-    my ($options) = @_;
+    my ( $self, $options ) = @_;
     return (
-              defined( $options->{box}{x} )
-          and defined( $options->{box}{y} )
-          and defined( $options->{box}{l} )
-          and defined( $options->{box}{t} )
+              defined( $self->{geometry_boxes}{x} )
+          and defined( $self->{geometry_boxes}{y} )
+          and defined( $self->{geometry_boxes}{l} )
+          and defined( $self->{geometry_boxes}{t} )
           and ( not defined $options->by_name(SANE_NAME_PAGE_HEIGHT)
-            or defined( $options->{box}{$SANE_NAME_PAGE_HEIGHT} ) )
+            or defined( $self->{geometry_boxes}{$SANE_NAME_PAGE_HEIGHT} ) )
           and ( not defined $options->by_name(SANE_NAME_PAGE_WIDTH)
-            or defined( $options->{box}{$SANE_NAME_PAGE_WIDTH} ) )
+            or defined( $self->{geometry_boxes}{$SANE_NAME_PAGE_WIDTH} ) )
     );
 }
 
@@ -600,7 +610,7 @@ sub create_paper_widget {
 
     # Only define the paper size once the rest of the geometry widgets
     # have been created
-    if (    _geometry_widgets_created($options)
+    if (    $self->_geometry_widgets_created($options)
         and not defined( $self->{combobp} )
         and defined( $self->{hboxp} ) )
     {
@@ -631,8 +641,8 @@ sub create_paper_widget {
                         )
                       )
                     {
-                        if ( defined $options->{box}{$_} ) {
-                            $options->{box}{$_}->show_all;
+                        if ( defined $self->{geometry_boxes}{$_} ) {
+                            $self->{geometry_boxes}{$_}->show_all;
                         }
                     }
                     $self->set( 'paper', undef );
@@ -650,7 +660,7 @@ sub create_paper_widget {
           )
         {
             if ( defined $options->by_name($_) ) {
-                my $widget = $options->by_name($_)->{widget};
+                my $widget = $self->{option_widgets}{$_};
                 $widget->signal_connect(
                     changed => sub {
                         if ( defined $self->get('paper') ) {
@@ -674,11 +684,17 @@ sub set_paper {
         return;
     }
     for ( @{ $self->{ignored_paper_formats} } ) {
-        if ( $_ eq $paper ) { return }
+        if ( $_ eq $paper ) {
+            if ( defined $logger ) {
+                $logger->info("Ignoring unsupported paper $paper");
+            }
+            return;
+        }
     }
     my $formats = $self->get('paper-formats');
     my $options = $self->get('available-scan-options');
-    my @paper_profile;
+    my $paper_profile;
+    $paper_profile->{backend} = [];
     if ( defined( $options->by_name(SANE_NAME_PAGE_HEIGHT) )
         and not $options->by_name(SANE_NAME_PAGE_HEIGHT)->{cap} &
         SANE_CAP_INACTIVE
@@ -687,25 +703,25 @@ sub set_paper {
         SANE_CAP_INACTIVE )
     {
         $self->build_profile(
-            \@paper_profile,
+            $paper_profile,
             $options->by_name(SANE_NAME_PAGE_HEIGHT),
             $formats->{$paper}{y} + $formats->{$paper}{t}
         );
         $self->build_profile(
-            \@paper_profile,
+            $paper_profile,
             $options->by_name(SANE_NAME_PAGE_WIDTH),
             $formats->{$paper}{x} + $formats->{$paper}{l}
         );
     }
     for (qw( l t x y )) {
         $self->build_profile(
-            \@paper_profile,
+            $paper_profile,
             $options->by_name($_),
             $formats->{$paper}{$_}
         );
     }
 
-    if ( not @paper_profile ) {
+    if ( not @{ $paper_profile->{backend} } ) {
         $self->hide_geometry($options);
         $self->{paper} = $paper;
         $self->signal_emit( 'changed-paper', $paper );
@@ -725,7 +741,7 @@ sub set_paper {
 
 # Don't trigger the changed-paper signal until we have finished setting the profile
     $self->{setting_profile} = TRUE;
-    $self->_set_option_profile( 0, \@paper_profile );
+    $self->_set_option_profile( 0, $paper_profile );
     return;
 }
 
@@ -733,7 +749,9 @@ sub hide_geometry {
     my ( $self, $options ) = @_;
     for ( ( 'l', 't', 'x', 'y', SANE_NAME_PAGE_HEIGHT, SANE_NAME_PAGE_WIDTH ) )
     {
-        if ( defined $options->{box}{$_} ) { $options->{box}{$_}->hide_all; }
+        if ( defined $self->{geometry_boxes}{$_} ) {
+            $self->{geometry_boxes}{$_}->hide_all;
+        }
     }
     return;
 }
@@ -830,7 +848,8 @@ sub set_option {
                 prefix   => $self->get('prefix'),
                 frontend => $self->get('frontend'),
                 device   => $self->get('device'),
-                options  => $self->map_options( $self->{current_scan_options} ),
+                options =>
+                  $self->map_options( $self->{current_scan_options}{backend} ),
                 started_callback => sub {
 
                     # Set up ProgressBar
@@ -923,7 +942,7 @@ sub patch_cache {
     # reference tree, parts of $self->{current_scan_options}
     # are undef
     Dumper( $self->{current_scan_options} );
-    for my $hashref ( @{ $self->{current_scan_options} } ) {
+    for my $hashref ( @{ $self->{current_scan_options}{backend} } ) {
         my ( $key, undef ) = each %{$hashref};
         my $updated_option =
           $self->get('available-scan-options')->by_name($key);
@@ -943,7 +962,7 @@ sub update_widget {    # FIXME: this is partly duplicated in Sane.pm
 
     my ( $group, $vbox );
     my $opt    = $self->get('available-scan-options')->by_name($name);
-    my $widget = $opt->{widget};
+    my $widget = $self->{option_widgets}{$name};
 
     # could be undefined for !($opt->{cap} & SANE_CAP_SOFT_DETECT)
     if ( defined $widget ) {
@@ -1010,54 +1029,53 @@ sub update_widget {    # FIXME: this is partly duplicated in Sane.pm
 # As scanimage and scanadf rename the geometry options,
 # we have to map them back to the original names
 sub map_geometry_names {
-    my ( $self, $profile ) = @_;
-    for my $i ( 0 .. $#{$profile} ) {
+    my ( $self, $options ) = @_;
+    for my $i ( 0 .. $#{$options} ) {
 
         # for reasons I don't understand, without walking the reference tree,
-        # parts of $profile are undef
-        Dumper($profile);
-        my ( $name, $val ) = each %{ $profile->[$i] };
+        # parts of $options are undef
+        Dumper($options);
+        my ( $name, $val ) = each %{ $options->[$i] };
         given ($name) {
             when (SANE_NAME_SCAN_TL_X) {
                 $name = 'l';
-                $profile->[$i] = { $name => $val };
+                $options->[$i] = { $name => $val };
             }
             when (SANE_NAME_SCAN_TL_Y) {
                 $name = 't';
-                $profile->[$i] = { $name => $val };
+                $options->[$i] = { $name => $val };
             }
             when (SANE_NAME_SCAN_BR_X) {
                 $name = 'x';
-                my $l = $self->get_option_from_profile( 'l', $profile );
+                my $l = $self->get_option_from_profile( 'l', $options );
                 if ( not defined $l ) {
                     $l =
                       $self->get_option_from_profile( SANE_NAME_SCAN_TL_X,
-                        $profile );
+                        $options );
                 }
                 if ( defined $l ) { $val -= $l }
-                $profile->[$i] = { $name => $val };
+                $options->[$i] = { $name => $val };
             }
             when (SANE_NAME_SCAN_BR_Y) {
                 $name = 'y';
-                my $t = $self->get_option_from_profile( 't', $profile );
+                my $t = $self->get_option_from_profile( 't', $options );
                 if ( not defined $t ) {
                     $t =
                       $self->get_option_from_profile( SANE_NAME_SCAN_TL_Y,
-                        $profile );
+                        $options );
                 }
                 if ( defined $t ) { $val -= $t }
-                $profile->[$i] = { $name => $val };
+                $options->[$i] = { $name => $val };
             }
         }
     }
     return;
 }
 
-# Remove paper size from options,
 # change boolean values from TRUE and FALSE to yes and no
 sub map_options {
     my ( $self, $old ) = @_;
-    my $new;
+    my @new;
     my $options = $self->get('available-scan-options');
     for ( @{$old} ) {
 
@@ -1065,15 +1083,13 @@ sub map_options {
         # parts of $_ are undef
         Dumper($_);
         my ( $key, $val ) = each %{$_};
-        if ( $key ne 'Paper size' ) {
-            my $opt = $options->by_name($key);
-            if ( defined( $opt->{type} ) and $opt->{type} == SANE_TYPE_BOOL ) {
-                $val = $val ? 'yes' : 'no';
-            }
-            push @{$new}, { $key => $val };
+        my $opt = $options->by_name($key);
+        if ( defined( $opt->{type} ) and $opt->{type} == SANE_TYPE_BOOL ) {
+            $val = $val ? 'yes' : 'no';
         }
+        push @new, { $key => $val };
     }
-    return $new;
+    return \@new;
 }
 
 sub scan {
@@ -1093,7 +1109,7 @@ sub scan {
 
     # As scanimage and scanadf rename the geometry options,
     # we have to map them back to the original names
-    my $options = $self->{current_scan_options};
+    my $options = $self->{current_scan_options}{backend};
     $self->map_geometry_names($options);
 
     my $i = 1;
