@@ -24,7 +24,7 @@ use Glib::Object::Subclass Gscan2pdf::Dialog::Scan::, properties => [
     ),
 ];
 
-our $VERSION = '1.5.0';
+our $VERSION = '1.5.1';
 
 my $SANE_NAME_SCAN_TL_X   = SANE_NAME_SCAN_TL_X;
 my $SANE_NAME_SCAN_TL_Y   = SANE_NAME_SCAN_TL_Y;
@@ -114,6 +114,10 @@ sub scan_options {
         $self->{notebook}->remove_page($LAST_PAGE);
     }
 
+    # Remove lookups to geometry boxes and option widgets
+    delete $self->{geometry_boxes};
+    delete $self->{option_widgets};
+
     # Ghost the scan button whilst options being updated
     if ( defined $self->{sbutton} ) { $self->{sbutton}->set_sensitive(FALSE) }
 
@@ -207,7 +211,7 @@ sub _initialise_options {    ## no critic (ProhibitExcessComplexity)
 
         # Define HBox for paper size here
         # so that it can be put before first geometry option
-        if ( $self->_geometry_option($opt) and not defined $hboxp ) {
+        if ( not defined $hboxp and $self->_geometry_option($opt) ) {
             $hboxp = Gtk2::HBox->new;
             $vbox->pack_start( $hboxp, FALSE, FALSE, 0 );
         }
@@ -355,14 +359,14 @@ sub create_paper_widget {
     # Only define the paper size once the rest of the geometry widgets
     # have been created
     if (
-            defined( $options->{box}{$SANE_NAME_SCAN_BR_X} )
-        and defined( $options->{box}{$SANE_NAME_SCAN_BR_Y} )
-        and defined( $options->{box}{$SANE_NAME_SCAN_TL_X} )
-        and defined( $options->{box}{$SANE_NAME_SCAN_TL_Y} )
+            defined( $self->{geometry_boxes}{$SANE_NAME_SCAN_BR_X} )
+        and defined( $self->{geometry_boxes}{$SANE_NAME_SCAN_BR_Y} )
+        and defined( $self->{geometry_boxes}{$SANE_NAME_SCAN_TL_X} )
+        and defined( $self->{geometry_boxes}{$SANE_NAME_SCAN_TL_Y} )
         and ( not defined $options->by_name(SANE_NAME_PAGE_HEIGHT)
-            or defined( $options->{box}{$SANE_NAME_PAGE_HEIGHT} ) )
+            or defined( $self->{geometry_boxes}{$SANE_NAME_PAGE_HEIGHT} ) )
         and ( not defined $options->by_name(SANE_NAME_PAGE_WIDTH)
-            or defined( $options->{box}{$SANE_NAME_PAGE_WIDTH} ) )
+            or defined( $self->{geometry_boxes}{$SANE_NAME_PAGE_WIDTH} ) )
         and not defined( $self->{combobp} )
       )
     {
@@ -394,8 +398,8 @@ sub create_paper_widget {
                         )
                       )
                     {
-                        if ( defined $options->{box}{$_} ) {
-                            $options->{box}{$_}->show_all;
+                        if ( defined $self->{geometry_boxes}{$_} ) {
+                            $self->{geometry_boxes}{$_}->show_all;
                         }
                     }
                     $self->set( 'paper', undef );
@@ -416,7 +420,7 @@ sub create_paper_widget {
           )
         {
             if ( defined $options->by_name($_) ) {
-                my $widget = $options->by_name($_)->{widget};
+                my $widget = $self->{option_widgets}{$_};
                 $widget->signal_connect(
                     changed => sub {
                         if ( defined $self->get('paper') ) {
@@ -440,11 +444,17 @@ sub set_paper {
         return;
     }
     for ( @{ $self->{ignored_paper_formats} } ) {
-        if ( $_ eq $paper ) { return }
+        if ( $_ eq $paper ) {
+            if ( defined $logger ) {
+                $logger->info("Ignoring unsupported paper $paper");
+            }
+            return;
+        }
     }
     my $formats = $self->get('paper-formats');
     my $options = $self->get('available-scan-options');
-    my @paper_profile;
+    my $paper_profile;
+    $paper_profile->{backend} = [];
     if ( defined( $options->by_name(SANE_NAME_PAGE_HEIGHT) )
         and not $options->by_name(SANE_NAME_PAGE_HEIGHT)->{cap} &
         SANE_CAP_INACTIVE
@@ -453,38 +463,38 @@ sub set_paper {
         SANE_CAP_INACTIVE )
     {
         $self->build_profile(
-            \@paper_profile,
+            $paper_profile,
             $options->by_name(SANE_NAME_PAGE_HEIGHT),
             $formats->{$paper}{y} + $formats->{$paper}{t}
         );
         $self->build_profile(
-            \@paper_profile,
+            $paper_profile,
             $options->by_name(SANE_NAME_PAGE_WIDTH),
             $formats->{$paper}{x} + $formats->{$paper}{l}
         );
     }
     $self->build_profile(
-        \@paper_profile,
+        $paper_profile,
         $options->by_name(SANE_NAME_SCAN_TL_X),
         $formats->{$paper}{l}
     );
     $self->build_profile(
-        \@paper_profile,
+        $paper_profile,
         $options->by_name(SANE_NAME_SCAN_TL_Y),
         $formats->{$paper}{t}
     );
     $self->build_profile(
-        \@paper_profile,
+        $paper_profile,
         $options->by_name(SANE_NAME_SCAN_BR_X),
         $formats->{$paper}{x} + $formats->{$paper}{l}
     );
     $self->build_profile(
-        \@paper_profile,
+        $paper_profile,
         $options->by_name(SANE_NAME_SCAN_BR_Y),
         $formats->{$paper}{y} + $formats->{$paper}{t}
     );
 
-    if ( not @paper_profile ) {
+    if ( not @{ $paper_profile->{backend} } ) {
         $self->hide_geometry($options);
         $self->{paper} = $paper;
         $self->signal_emit( 'changed-paper', $paper );
@@ -504,7 +514,7 @@ sub set_paper {
 
 # Don't trigger the changed-paper signal until we have finished setting the profile
     $self->{setting_profile} = TRUE;
-    $self->_set_option_profile( 0, \@paper_profile );
+    $self->_set_option_profile( 0, $paper_profile );
     return;
 }
 
@@ -517,7 +527,9 @@ sub hide_geometry {
         )
       )
     {
-        if ( defined $options->{box}{$_} ) { $options->{box}{$_}->hide_all; }
+        if ( defined $self->{geometry_boxes}{$_} ) {
+            $self->{geometry_boxes}{$_}->hide_all;
+        }
     }
     return;
 }
@@ -608,43 +620,43 @@ sub set_option {
 # As scanimage and scanadf rename the geometry options,
 # we have to map them back to the original names
 sub map_geometry_names {
-    my ( $self, $profile ) = @_;
-    for my $i ( 0 .. $#{$profile} ) {
+    my ( $self, $options ) = @_;
+    for my $i ( 0 .. $#{$options} ) {
 
         # for reasons I don't understand, without walking the reference tree,
-        # parts of $profile are undef
-        Dumper( $profile->[$i] );
-        my ( $name, $val ) = each %{ $profile->[$i] };
+        # parts of $options are undef
+        Dumper( $options->[$i] );
+        my ( $name, $val ) = each %{ $options->[$i] };
         given ($name) {
             when ('l') {
                 $name = SANE_NAME_SCAN_TL_X;
-                $profile->[$i] = { $name => $val };
+                $options->[$i] = { $name => $val };
             }
             when ('t') {
                 $name = SANE_NAME_SCAN_TL_Y;
-                $profile->[$i] = { $name => $val };
+                $options->[$i] = { $name => $val };
             }
             when ('x') {
                 $name = SANE_NAME_SCAN_BR_X;
-                my $l = $self->get_option_from_profile( 'l', $profile );
+                my $l = $self->get_option_from_profile( 'l', $options );
                 if ( not defined $l ) {
                     $l =
                       $self->get_option_from_profile( SANE_NAME_SCAN_TL_X,
-                        $profile );
+                        $options );
                 }
                 if ( defined $l ) { $val += $l; }
-                $profile->[$i] = { $name => $val };
+                $options->[$i] = { $name => $val };
             }
             when ('y') {
                 $name = SANE_NAME_SCAN_BR_Y;
-                my $t = $self->get_option_from_profile( 't', $profile );
+                my $t = $self->get_option_from_profile( 't', $options );
                 if ( not defined $t ) {
                     $t =
                       $self->get_option_from_profile( SANE_NAME_SCAN_TL_Y,
-                        $profile );
+                        $options );
                 }
                 if ( defined $t ) { $val += $t; }
-                $profile->[$i] = { $name => $val };
+                $options->[$i] = { $name => $val };
             }
         }
     }
