@@ -32,7 +32,7 @@ BEGIN {
     use Exporter ();
     our ( $VERSION, @EXPORT_OK, %EXPORT_TAGS );
 
-    $VERSION = '1.5.0';
+    $VERSION = '1.5.1';
 
     use base qw(Exporter);
     %EXPORT_TAGS = ();    # eg: TAG => [ qw!name1 name2! ],
@@ -385,7 +385,7 @@ sub _pdftotext2boxes {
 
 sub round {
     my ($f) = @_;
-    return int( $f + 0.5 );
+    return int( $f + 0.5 );    ## no critic (ProhibitMagicNumbers)
 }
 
 # return hocr output as string
